"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const electron_1 = require("electron");
const log_1 = require("./log");
const account_switcher_1 = require("./account_switcher");
const default_menu_1 = require("./default_menu");
const window_1 = require("./window");
const common_1 = require("./common");
class App {
    constructor(win, config) {
        this.win = win;
        this.config = config;
        this.toggleNormalWindow = () => {
            const win = this.win.browser;
            if (win.isFocused()) {
                log_1.default.debug('Toggle window: shown -> hidden');
                if (common_1.IS_DARWIN) {
                    electron_1.app.hide();
                }
                else {
                    win.hide();
                }
            }
            else {
                log_1.default.debug('Toggle window: hidden -> shown');
                win.show();
            }
        };
        this.onAccountSwitch = (next) => {
            this.win.close();
            window_1.default.create(next, this.config, this.win.menubar).then(win => {
                log_1.default.debug('Window was recreated again', next);
                this.win = win;
                this.start();
            });
        };
        if (config.accounts.length === 0) {
            throw new Error('No account found. Please check the config.');
        }
        if (common_1.IS_DARWIN) {
            electron_1.app.dock.setIcon(common_1.APP_ICON);
        }
        electron_1.Menu.setApplicationMenu(default_menu_1.default());
        this.switcher = new account_switcher_1.default(this.config.accounts);
        this.switcher.on('switch', this.onAccountSwitch);
        this.setupTray();
        this.setupHotkey();
    }
    setupHotkey() {
        if (!this.config.hot_key) {
            return;
        }
        if (this.win.menubar) {
            electron_1.globalShortcut.register(this.config.hot_key, () => {
                const mb = this.win.menubar;
                if (mb.window.isFocused()) {
                    log_1.default.debug('Toggle window: shown -> hidden');
                    mb.hideWindow();
                }
                else {
                    log_1.default.debug('Toggle window: hidden -> shown');
                    mb.showWindow();
                }
            });
        }
        else {
            electron_1.globalShortcut.register(this.config.hot_key, this.toggleNormalWindow);
        }
        log_1.default.debug('Hot key was set to:', this.config.hot_key);
    }
    start() {
        const a = this.switcher.current;
        const url = `https://${a.host}${a.default_page}`;
        this.win.open(url);
        log_1.default.debug('Application started', a, url);
    }
    setupTray() {
        if (this.win.menubar) {
            return;
        }
        const icon = common_1.trayIcon(this.config.icon_color);
        const tray = new electron_1.Tray(icon);
        tray.on('click', this.toggleNormalWindow);
        tray.on('double-click', this.toggleNormalWindow);
        if (common_1.IS_DARWIN) {
            tray.setHighlightMode('never');
        }
    }
}
exports.App = App;
function startApp(config) {
    const default_account = config.accounts[0];
    return window_1.default.create(default_account, config)
        .then(win => new App(win, config));
}
exports.default = startApp;
//# sourceMappingURL=app.js.map