"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const electron_1 = require("electron");
const fs = require("fs");
const log_1 = require("./log");
const common_1 = require("./common");
function makeDefaultConfig() {
    const IsDarkMode = (process.platform === 'darwin') && electron_1.systemPreferences.isDarkMode();
    const menubarBroken = process.platform === 'win32';
    return {
        hot_key: 'CmdOrCtrl+Shift+S',
        icon_color: IsDarkMode ? 'white' : 'black',
        always_on_top: false,
        normal_window: menubarBroken,
        zoom_factor: 0.9,
        accounts: [{
                name: '',
                host: '',
                default_page: '/web/timelines/home',
            }],
        keymaps: {
            j: 'scroll-down',
            k: 'scroll-up',
            i: 'scroll-top',
            m: 'scroll-bottom',
            1: '/web/statuses/new',
            2: '/web/timelines/home',
            3: '/web/notifications',
            4: '/web/timelines/public/local',
            5: '/web/timelines/public',
            6: '/web/getting-started'
        },
    };
}
function showDyingDialog(title, detail) {
    electron_1.dialog.showMessageBox({
        type: 'info',
        message: title,
        detail,
    }, () => {
        electron_1.app.quit();
    });
}
function recommendConfigAndDie(file) {
    const title = 'Please write configuration in JSON';
    const detail = 'You need to write up name and host in first item of accounts. Restart this app after writing up them. Please see README for more detail: https://github.com/rhysd/Mstdn#readme';
    electron_1.shell.openItem(file);
    showDyingDialog(title, detail);
}
function loadConfig() {
    return new Promise(resolve => {
        fs.readFile(common_1.CONFIG_FILE, 'utf8', (err, json) => {
            if (err) {
                log_1.default.info('Configuration file was not found, will create:', common_1.CONFIG_FILE);
                const default_config = makeDefaultConfig();
                if (electron_1.app.isReady()) {
                    fs.writeFileSync(common_1.CONFIG_FILE, JSON.stringify(default_config, null, 2));
                    recommendConfigAndDie(common_1.CONFIG_FILE);
                }
                else {
                    electron_1.app.once('ready', () => {
                        fs.writeFileSync(common_1.CONFIG_FILE, JSON.stringify(default_config, null, 2));
                        recommendConfigAndDie(common_1.CONFIG_FILE);
                    });
                }
                return;
            }
            try {
                const config = JSON.parse(json);
                if (config.hot_key && config.hot_key.startsWith('mod+')) {
                    config.hot_key = `CmdOrCtrl+${config.hot_key.slice(4)}`;
                }
                log_1.default.debug('Configuration was loaded successfully', config);
                if (!config.accounts || config.accounts[0].host === '' || config.accounts[0].name === '') {
                    recommendConfigAndDie(common_1.CONFIG_FILE);
                }
                else {
                    resolve(config);
                }
            }
            catch (e) {
                log_1.default.debug('Error on loading JSON file', e);
                showDyingDialog('Error on loading JSON file', e.message);
            }
        });
    });
}
exports.default = loadConfig;
//# sourceMappingURL=config.js.map