"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Mousetrap = require("mousetrap");
const Ipc = require("./ipc");
const log_1 = require("./log");
function scrollable() {
    const scrollable = document.querySelector('.scrollable');
    if (!scrollable) {
        log_1.default.error('Scrollable element was not found!');
        return { scrollTop: 0 };
    }
    return scrollable;
}
function navigateTo(host, path) {
    const url = `https://${host}${path}`;
    if (window.location.href === url) {
        log_1.default.info('Current URL is already', url);
        return;
    }
    const link = document.querySelector(`a[href="${path}"]`);
    if (link) {
        log_1.default.info('Click link by shortcut', path);
        link.click();
    }
    else {
        log_1.default.info('Force navigation by shortcut', path);
        window.location.href = url;
    }
}
const ShortcutActions = {
    'scroll-top': () => {
        scrollable().scrollTop = 0;
    },
    'scroll-bottom': () => {
        scrollable().scrollTop = document.body.scrollHeight;
    },
    'scroll-down': () => {
        scrollable().scrollTop += window.innerHeight / 3;
    },
    'scroll-up': () => {
        scrollable().scrollTop -= window.innerHeight / 3;
    },
    'next-account': () => {
        Ipc.send('mstdn:next-account');
    },
    'prev-account': () => {
        Ipc.send('mstdn:prev-account');
    },
};
function setupKeybinds(keybinds, host) {
    for (const key in keybinds) {
        const action = keybinds[key];
        if (action.startsWith('/')) {
            Mousetrap.bind(key, e => {
                e.preventDefault();
                navigateTo(host, action);
            });
        }
        else {
            const func = ShortcutActions[action];
            if (!func) {
                log_1.default.error('Unknown shortcut action:', action);
                continue;
            }
            Mousetrap.bind(key, e => {
                log_1.default.info('Shortcut:', action);
                e.preventDefault();
                func();
            });
        }
    }
}
let config = null;
Ipc.on('mstdn:config', (c, a) => {
    config = c;
    setupKeybinds(config.keymaps, a.host);
});
//# sourceMappingURL=index.js.map